# RiseSet.py
""" Reads data from  files that contain sunrise and sunset times
throughout the year.
"""

from numpy import *

def ConvertTime(s):
    """ Returns a float that is the equivalent (in hours)
    of the time encoded by s.
    
    '21:45' means 9:45 pm.
    
    PredC: s a 5-character string of the form hh:mm that
    specifies time.
    """
    
    return float(s[0:2])+float(s[2:])/60

def LatLong(s):
    """ Returns a tuple (Long,Lat) of floats that are the
    equivalent (in degrees) of the longitude and latitude
    encoded by s.
  
    
    PredC: s an 11-character string of the form 'cdddmmCDDMM'
    where cdddmm specifies longitude in degrees and minutes with
    c =  'W' or 'E' and CDDMM species latitude in degrees and minutes
    with C = 'N' or 'S'
    """
    Long = float(s[1:4])+float(s[4:6])/60
    if s[0]=='E':
       Long = -Long
    Lat = float(s[7:9])+float(s[9:11])/60
    if s[6]=='S':
        Lat = -Lat
    return (Lat,Long)

    
class Daylight:
    """
    Attributes:
       City:     name of the city [str]
       Lat:      latitude in degrees [float]
       Long:     longitude in degrees [float]
       RiseTime: length-365 numpy array of sunrise times
       SetTime:  length-365 numpy array of sunset times

    For k in range(365), RiseTime[k] is the sunrise time on the k-th day of the
    year and RiseTime[k] is the sunset time on the kth day of the year,
    """
    
    def __init__(self,FileName):
        """ Returns a Daylight object
        
        PreC: Filename is a path to a rise/set file [str]
        """
      
        self.RiseTime = zeros(365)
        self.SetTime  = zeros(365)
        f = file(FileName, 'r')
        lineNum = 0
        for line in f:
           # Isolate the columns of data
           parts = line.split()
           lineNum+=1
           if lineNum==1:
                # First line has the city name
                self.City = parts[0]
           elif lineNum==2:
                # Second line has lat/long information
                (self.Lat,self.Long) = LatLong(parts[0])
           else:
                # Remaining lines have rise/set pairs 
                day = int(parts[0])
                # Get all the rise and set times
                RiseTimeList = map(ConvertTime,parts[1:len(parts):2])
                SetTimeList  = map(ConvertTime,parts[2:len(parts):2])
                p = len(RiseTimeList)
                for k in range(p):
                    if day<=28:
                        # All months have at least 28 days
                        starts  = [0,31,59,90,120,151,181,212,243,273,304,334]
                        dayIndex = day + starts[k] - 1
                    elif day==29 or day==30:
                        # All months except February have a day 29 and a day 30
                        starts = [0, 59,90,120,151,181,212,243,273,304,334]
                        dayIndex = day + starts[k] - 1
                    else:
                        # Only January, March, May, July, August, October, and December have
                        # a day 31.
                        starts = [0,59,120,181,212,273,334]
                        dayIndex = day + starts[k] - 1
                    self.RiseTime[dayIndex] = RiseTimeList[k]
                    self.SetTime[dayIndex] = SetTimeList[k]
        f.close()
                    
    def SunUp(self):
        """ Returns a length-365 numpy array D with the
        property that D[k] is the sun-up time on day k.
        """
        return self.SetTime - self.RiseTime
    
    def MonthAves(self):
        """ Returns a length-12 numpy array M with the
        property that M[k] is the average sun-up time
        during month k.
        """
        M = zeros(12)
        D = self.SunUp()
        start  = [0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334,365]
        for k in range(12):
            z = D[start[k]:start[k+1]]
            M[k] = sum(z)/len(z)
        return M          
              
if __name__ == '__main__':
    # There is Rise/Set data for these cities..
    Cities = (['Anaheim','Anchorage','Arlington','Athens','Atlanta',
      'Baltimore','Bangkok','Beijing','Berlin','Bogata',
      'Boston','BuenosAires','Cairo','Chicago','Cincinnati',
      'Cleveland','Denver','Detroit','Honolulu','Houston',
      'Ithaca','Johannesburg','KansasCity','Lagos','London',
      'LosAngeles','MexicoCity','Miami','Milwaukee','Minneapolis',
      'Moscow','NewDelhi','NewYork','Oakland','Paris',
      'Philadelphia','Phoenix','Pittsburgh','RiodeJaneiro','Rome', 
      'SanFrancisco','Seattle','Seoul','Sydney','Tampa',
      'Teheran','Tokyo','Toronto','Washington','Wellington'])
    
    print '              City     Latitude    June 22    Dec 22'
    print '-------------------------------------------------------'
    for C in Cities:
        FN = 'RiseSetData/' + C + '.dat'
        A = Daylight(FN)
        D = A.SunUp()
        L = A.Lat
        M = A.MonthAves()
        print '%20s  %6.0f       %6.2f    %6.2f  ' % (C,L,D[173],D[356])
        
    ESTCities = (['Boston','NewYork',
                  'Philadelphia','Ithaca','Baltimore','Washington',
      'Toronto','Pittsburgh','Miami','Tampa','Cleveland',
      'Detroit','Atlanta','Cincinnati'])
    
    print '\n\n              City     Longitude     Sunrise Mar 22  '
    print '-------------------------------------------------------'
    for C in ESTCities:
        FN = 'RiseSetData/' + C + '.dat'
        A = Daylight(FN)
        T = A.RiseTime[81]
        h = floor(T)
        m = floor((T-h)*60)
    
        print '%20s  %6.0f             %2dh %2dm    ' % (C,A.Long,h,m)
    

    
 
    
